@@ -37,7 +37,7 @@ Example usage for Google OpenID::
     class GoogleLoginHandler(tornado.web.RequestHandler,
                              tornado.auth.GoogleMixin):
         @tornado.web.asynchronous
-        @tornado.gen.coroutine
+        @tornado.gen.engine
         def get(self):
             if self.get_argument("openid.mode", None):
                 user = yield self.get_authenticated_user()
@@ -572,7 +572,7 @@ class TwitterMixin(OAuthMixin):
         class TwitterLoginHandler(tornado.web.RequestHandler,
                                   tornado.auth.TwitterMixin):
             @tornado.web.asynchronous
-            @tornado.gen.coroutine
+            @tornado.gen.engine
             def get(self):
                 if self.get_argument("oauth_token", None):
                     user = yield self.get_authenticated_user()
@@ -585,12 +585,12 @@ class TwitterMixin(OAuthMixin):
     and all of the custom Twitter user attributes described at
     https://dev.twitter.com/docs/api/1.1/get/users/show
     """
-    _OAUTH_REQUEST_TOKEN_URL = "http://api.twitter.com/oauth/request_token"
-    _OAUTH_ACCESS_TOKEN_URL = "http://api.twitter.com/oauth/access_token"
-    _OAUTH_AUTHORIZE_URL = "http://api.twitter.com/oauth/authorize"
-    _OAUTH_AUTHENTICATE_URL = "http://api.twitter.com/oauth/authenticate"
+    _OAUTH_REQUEST_TOKEN_URL = "https://api.twitter.com/oauth/request_token"
+    _OAUTH_ACCESS_TOKEN_URL = "https://api.twitter.com/oauth/access_token"
+    _OAUTH_AUTHORIZE_URL = "https://api.twitter.com/oauth/authorize"
+    _OAUTH_AUTHENTICATE_URL = "https://api.twitter.com/oauth/authenticate"
     _OAUTH_NO_CALLBACKS = False
-    _TWITTER_BASE_URL = "http://api.twitter.com/1"
+    _TWITTER_BASE_URL = "https://api.twitter.com/1.1"
 
     def authenticate_redirect(self, callback_uri=None):
         """Just like `~OAuthMixin.authorize_redirect`, but
@@ -627,7 +627,7 @@ class TwitterMixin(OAuthMixin):
                               tornado.auth.TwitterMixin):
                 @tornado.web.authenticated
                 @tornado.web.asynchronous
-                @tornado.gen.coroutine
+                @tornado.gen.engine
                 def get(self):
                     new_entry = yield self.twitter_request(
                         "/statuses/update",
@@ -706,7 +706,7 @@ class FriendFeedMixin(OAuthMixin):
         class FriendFeedLoginHandler(tornado.web.RequestHandler,
                                      tornado.auth.FriendFeedMixin):
             @tornado.web.asynchronous
-            @tornado.gen.coroutine
+            @tornado.gen.engine
             def get(self):
                 if self.get_argument("oauth_token", None):
                     user = yield self.get_authenticated_user()
@@ -751,7 +751,7 @@ class FriendFeedMixin(OAuthMixin):
                               tornado.auth.FriendFeedMixin):
                 @tornado.web.authenticated
                 @tornado.web.asynchronous
-                @tornado.gen.coroutine
+                @tornado.gen.engine
                 def get(self):
                     new_entry = yield self.friendfeed_request(
                         "/entry",
@@ -835,7 +835,7 @@ class GoogleMixin(OpenIdMixin, OAuthMixin):
         class GoogleLoginHandler(tornado.web.RequestHandler,
                                  tornado.auth.GoogleMixin):
            @tornado.web.asynchronous
-           @tornado.gen.coroutine
+           @tornado.gen.engine
            def get(self):
                if self.get_argument("openid.mode", None):
                    user = yield self.get_authenticated_user()
@@ -1105,7 +1105,7 @@ class FacebookGraphMixin(OAuth2Mixin):
 
             class FacebookGraphLoginHandler(LoginHandler, tornado.auth.FacebookGraphMixin):
               @tornado.web.asynchronous
-              @tornado.gen.coroutine
+              @tornado.gen.engine
               def get(self):
                   if self.get_argument("code", False):
                       user = yield self.get_authenticated_user(
@@ -1192,7 +1192,7 @@ class FacebookGraphMixin(OAuth2Mixin):
                               tornado.auth.FacebookGraphMixin):
                 @tornado.web.authenticated
                 @tornado.web.asynchronous
-                @tornado.gen.coroutine
+                @tornado.gen.engine
                 def get(self):
                     new_entry = yield self.facebook_request(
                         "/me/feed",
@@ -1223,7 +1223,7 @@ class FacebookGraphMixin(OAuth2Mixin):
 
     def _on_facebook_request(self, future, response):
         if response.error:
-            future.set_exception(AuthError("Error response %s fetching %s", 
+            future.set_exception(AuthError("Error response %s fetching %s",
                                     response.error, response.request.url))
             return
 
